@@ -5,20 +5,20 @@
 modname = 'cubicweb_jsonschema'
 distname = 'cubicweb-jsonschema'
 
-numversion = (0, 2, 11)
+numversion = (0, 5, 0)
 version = '.'.join(str(num) for num in numversion)
 
 license = 'LGPL'
 author = 'LOGILAB S.A. (Paris, FRANCE)'
 author_email = 'contact@logilab.fr'
 description = 'JSON Schema for CubicWeb'
-web = 'https://www.cubicweb.org/project/%s' % distname
+web = 'http://www.cubicweb.org/project/%s' % distname
 
 __depends__ = {
-    'cubicweb': '>= 3.37.2, < 3.39.0',
+    'cubicweb[pyramid]': '>= 3.24.1',
+    'six': '>= 1.4.0',
     'iso8601': None,
-    'jsl': None,
-    'pyramid': '>= 1.10.8',
+    'pyramid': '>= 1.8.0',
 }
 __recommends__ = {}
 
@@ -26,7 +26,5 @@ classifiers = [
     'Environment :: Web Environment',
     'Framework :: CubicWeb',
     'Programming Language :: Python',
-    'Programming Language :: Python :: 3',
-    'Programming Language :: Python :: 3 :: Only',
     'Programming Language :: JavaScript',
 ]
