@@ -12,11 +12,15 @@
 # details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with this program. If not, see <https://www.gnu.org/licenses/>.
-"""cubicweb-jsonschema yams constraints to jsl serializers"""
+# along with this program. If not, see <http://www.gnu.org/licenses/>.
+"""cubicweb-jsonschema mappers for Yams constraints."""
 
 from logilab.common.registry import Predicate
 
+from .. import (
+    CREATION_ROLE,
+    EDITION_ROLE,
+)
 from . import JSONSchemaMapper
 
 
@@ -29,47 +33,43 @@ class yams_constraint_type(Predicate):
     def __init__(self, cstr_type):
         self.cstr_type = cstr_type
 
-    def __call__(self, cls, vreg, _, etype, rtype, cstr, jsl_attrs):
+    def __call__(self, cls, vreg, _, etype, rtype, cstr, **kwargs):
         if cstr.type() == self.cstr_type:
             return 1
         return 0
 
 
-class JslConstraintSerializer(JSONSchemaMapper):
+class ConstraintSerializer(JSONSchemaMapper):
     __regid__ = 'jsonschema.constraint'
     __abstract__ = True
 
-    def __init__(self, vreg, _, etype, rtype, constraint, jsl_attrs):
+    def __init__(self, vreg, _, etype, rtype, constraint):
         self.vreg = vreg
         self._ = _
         self.etype = vreg.schema[etype]
         self.rtype = vreg.schema[rtype]
         self.constraint = constraint
-        self.jsl_attrs = jsl_attrs
-
-    def todict(self):
-        raise NotImplementedError
 
 
-class JslSizeConstraintSerializer(JslConstraintSerializer):
+class SizeConstraintSerializer(ConstraintSerializer):
     __select__ = yams_constraint_type('SizeConstraint')
 
-    def todict(self):
-        if ('min_length' not in self.jsl_attrs
-                and self.constraint.min is not None):
-            return {'min_length': self.constraint.min}
-        if ('max_length' not in self.jsl_attrs
-                and self.constraint.max is not None):
-            return {'max_length': self.constraint.max}
+    def schema_and_definitions(self, schema_role=None):
+        if schema_role not in (CREATION_ROLE, EDITION_ROLE):
+            return {}, None
+        if self.constraint.min is not None:
+            return {'minLength': self.constraint.min}, None
+        if self.constraint.max is not None:
+            return {'maxLength': self.constraint.max}, None
 
 
-class JslIntervalBoundConstraintSerializer(JslConstraintSerializer):
+class IntervalBoundConstraintSerializer(ConstraintSerializer):
     __select__ = yams_constraint_type('IntervalBoundConstraint')
 
-    def todict(self):
-        if ('minimum' not in self.jsl_attrs
-                and self.constraint.minvalue is not None):
-            return {'minimum': self.constraint.minvalue}
-        if ('maximum' not in self.jsl_attrs
-                and self.constraint.maxvalue is not None):
-            return {'maximum': self.constraint.maxvalue}
+    def schema_and_definitions(self, schema_role=None):
+        if schema_role not in (CREATION_ROLE, EDITION_ROLE):
+            return {}, None
+        if self.constraint.minvalue is not None:
+            return {'minimum': self.constraint.minvalue}, None
+        if self.constraint.maxvalue is not None:
+            return {'maximum': self.constraint.maxvalue}, None
