@@ -12,7 +12,7 @@
 # details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with this program. If not, see <https://www.gnu.org/licenses/>.
+# along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 """cubicweb-jsonschema predicates"""
 
@@ -25,9 +25,14 @@ class NoWildcardAcceptPredicate(predicates.AcceptPredicate):
     See https://github.com/Pylons/pyramid/issues/1264.
     """
 
+    def __init__(self, val, config):
+        if not isinstance(val, (list, tuple)):
+            val = (val, )
+        super(NoWildcardAcceptPredicate, self).__init__(val, config)
+
     def __call__(self, context, request):
         request_accepts = [h for h in request.accept if h != '*/*']
-        for accept_type in self.values:
+        for accept_type in self.val:
             if accept_type in request_accepts:
                 return True
         return False
