@@ -1,23 +1,8 @@
-# copyright 2016-2021 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
-# contact https://www.logilab.fr -- mailto:contact@logilab.fr
-#
-# This program is free software: you can redistribute it and/or modify it under
-# the terms of the GNU Lesser General Public License as published by the Free
-# Software Foundation, either version 2.1 of the License, or (at your option)
-# any later version.
-#
-# This program is distributed in the hope that it will be useful, but WITHOUT
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
-# details.
-#
-# You should have received a copy of the GNU Lesser General Public License
-# along with this program. If not, see <https://www.gnu.org/licenses/>.
-
 """doctest utilities"""
 
 from collections import defaultdict
 import doctest
+from functools import partial
 import json
 import re
 import unittest
@@ -27,6 +12,7 @@ from jsondiff.symbols import (
     delete,
     insert,
 )
+from six import string_types
 
 
 class ParseError(Exception):
@@ -71,10 +57,10 @@ def _parse_webtest_testresponse(string):
             break
     else:
         raise ParseError('not a webtest.TestResponse string')
-    if not re.match(r'Response: \d{3} \w+', status):
+    if not re.match('Response: \d{3} \w+', status):
         raise ParseError('not a webtest.TestResponse string')
     headers = []
-    while re.match(r'[a-zA-Z\-]+: [^\n]+\n', rest):
+    while re.match('[a-zA-Z\-]+: [^\n]+\n', rest):
         header, rest = rest.split('\n', 1)
         name, value = header.split(':', 1)
         headers.append((name.strip(), value.strip()))
@@ -160,7 +146,7 @@ def jsondiff_filter_ellipsis(diffobj):
                         break
                 else:
                     del diffobj[key]
-            elif isinstance(want, str):
+            elif isinstance(want, string_types):
                 if doctest._ellipsis_match(want, got):
                     del diffobj[key]
 
@@ -195,6 +181,9 @@ def _headerlist_to_string(headerlist):
                      for name, value in headerlist)
 
 
+WEBTEST_RESPONSE = doctest.register_optionflag('WEBTEST_RESPONSE')
+
+
 class WebTestReponseOutputChecker(doctest.OutputChecker):
     """Output checker for webtest.TestResponse objects."""
 
@@ -202,6 +191,8 @@ class WebTestReponseOutputChecker(doctest.OutputChecker):
         # No super() since doctest.OutputChecker is an old-style class.
         if doctest.OutputChecker.check_output(self, want, got, optionflags):
             return True
+        if not optionflags & WEBTEST_RESPONSE:
+            return False
         try:
             w_status, w_headers, w_body = _parse_webtest_testresponse(want)
             g_status, g_headers, g_body = _parse_webtest_testresponse(got)
@@ -256,6 +247,9 @@ class WebTestReponseOutputChecker(doctest.OutputChecker):
         return diff
 
     def output_difference(self, example, got, optionflags):
+        if not optionflags & WEBTEST_RESPONSE:
+            return doctest.OutputChecker.output_difference(
+                self, example, got, optionflags)
         want = example.want
         errors = []
         try:
@@ -308,9 +302,10 @@ class WebTestReponseOutputChecker(doctest.OutputChecker):
                             'actual\n{}',
                             'diff:\n{}'
                         ])
-                        diff.append(msg.format(json.dumps(w_json, indent=2),
-                                               json.dumps(g_json, indent=2),
-                                               json.dumps(body_diff)))
+                        dumps = partial(json.dumps, indent=2, sort_keys=True)
+                        diff.append(msg.format(dumps(w_json),
+                                               dumps(g_json),
+                                               dumps(body_diff)))
         return '\n'.join(diff)
 
 
