@@ -1,4 +1,4 @@
-# copyright 2016-2022 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+# copyright 2016 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 # contact http://www.logilab.fr -- mailto:contact@logilab.fr
 #
 # This program is free software: you can redistribute it and/or modify it under
@@ -12,7 +12,7 @@
 # details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with this program. If not, see <https://www.gnu.org/licenses/>.
+# along with this program. If not, see <http://www.gnu.org/licenses/>.
 """cubicweb-jsonschema views/forms/actions/components for web ui"""
 
 from cubicweb.__pkginfo__ import numversion as cw_version
@@ -70,7 +70,7 @@ class JSONSchemaSectionRelationTags(RelationTags):
         else:
             relpermission = permission
         eid = entity.eid if entity.has_eid() else None
-        rdefs = eschema.relation_definitions(include_final=True)
+        rdefs = eschema.relation_definitions(includefinal=True)
         for rschema, targetschemas, role in rdefs:
             if rschema in self.ignored_rtypes:
                 continue
@@ -85,12 +85,12 @@ class JSONSchemaSectionRelationTags(RelationTags):
                 if section not in self.etype_get(eschema, rschema, role,
                                                  tschema):
                     continue
-                rdef = rschema.role_relation_definition(eschema, tschema, role)
+                rdef = rschema.role_rdef(eschema, tschema, role)
                 if rschema.final:
                     if not rdef.has_perm(cnx, permission, eid=eid,
                                          creating=eid is None):
                         continue
-                else:
+                elif not rdef.has_local_role(relpermission):
                     if not rdef.has_perm(cnx, relpermission, **relperms_kwargs):
                         continue
                 target_types.add(tschema.type)
