@@ -5,14 +5,13 @@ from yams.buildobjs import (
     ComputedRelation,
     Date,
     Float,
-    Int,
     RelationDefinition,
     String,
-    RichString,
 )
 from cubicweb.schema import (
     ERQLExpression,
     RRQLExpression,
+    WorkflowableEntityType,
 )
 
 
@@ -78,7 +77,17 @@ class author(RelationDefinition):
     }
     subject = 'Book'
     object = 'Author'
-    cardinality = '?*'
+    cardinality = '1*'
+    composite = 'object'
+
+
+class Library(EntityType):
+    name = String(required=True)
+
+
+class in_library(RelationDefinition):
+    subject = 'Book'
+    object = 'Library'
 
 
 class publications(ComputedRelation):
@@ -99,3 +108,7 @@ class Topic(EntityType):
 class topics(RelationDefinition):
     subject = 'Book'
     object = 'Topic'
+
+
+class UserAccount(WorkflowableEntityType):
+    username = String(required=True, unique=True)
