@@ -3,9 +3,11 @@ from cubicweb.web.views import uicfg
 from cubicweb_jsonschema.views import jsonschema_section
 
 
-
+uicfg.indexview_etype_section['Book'] = 'application'
+uicfg.indexview_etype_section['Library'] = 'subobject'
 uicfg.indexview_etype_section['Photo'] = 'subobject'
 uicfg.indexview_etype_section['Topic'] = 'subobject'
+uicfg.indexview_etype_section['UserAccount'] = 'subobject'
 
 jsonschema_section.tag_attribute(
     ('CWUser', 'lastname'), 'hidden')
