@@ -12,24 +12,59 @@
 # details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with this program. If not, see <https://www.gnu.org/licenses/>.
+# along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 """cubicweb-jsonschema unit tests for Pyramid JSON API."""
 
 import base64
 from datetime import date
-from unittest import skip
+import json
+from unittest import (
+    TestCase,
+    skip,
+)
 
 import jsonschema
 from mock import patch
+from pyramid import (
+    renderers,
+    testing,
+)
+import six
+from webob.multidict import MultiDict
 
 from cubicweb import Binary, ValidationError
 from cubicweb.pyramid.test import PyramidCWTest
 
 from cubicweb_jsonschema import VIEW_ROLE
-from cubicweb_jsonschema.entities.ijsonschema import (
-    jsonschema_adapter,
-)
+from cubicweb_jsonschema.links import parse_links
+
+
+class JSONSchemaRendererTC(TestCase):
+
+    def setUp(self):
+        self.config = testing.setUp()
+        self.config.include('cubicweb_jsonschema.api.schema')
+
+    def test_metaschema_for_object(self):
+        self.assertEqual(
+            renderers.render('jsonschema', {}),
+            '{"$schema": "http://json-schema.org/draft-06/schema#"}',
+        )
+        self.assertEqual(
+            json.loads(renderers.render('jsonschema', {'type': 'string'})),
+            {'$schema': 'http://json-schema.org/draft-06/schema#',
+             'type': 'string'},
+        )
+
+    def test_no_metaschema_for_bool(self):
+        for value in (False, True):
+            self.assertEqual(renderers.render('jsonschema', value),
+                             str(value).lower())
+
+    def test_invalid_value(self):
+        with self.assertRaises(ValueError):
+            renderers.render('jsonschema', 'a')
 
 
 class BaseTC(PyramidCWTest):
@@ -90,61 +125,123 @@ class EntitiesTC(BaseTC):
             book = cnx.create_entity('Book', title=u'title')
             author = cnx.create_entity('Author', name=u'bob',
                                        reverse_author=book)
-            jschema = self.vreg['adapters'].select(
-                'IJSONSchema', cnx, entity=author,
-                rtype='author', role='object').view_schema()
             cnx.commit()
+        self.login('anon', 'anon')
         url = '/book/%s/author' % book.eid
         res = self.webapp.get(url, headers={'accept': 'application/json'})
         related = res.json
         expected = [{
-            'type': u'author',
             'id': str(author.eid),
-            'title': u'bob',
         }]
         with self.admin_access.cnx() as cnx:
             collection_mapper = cnx.vreg['mappers'].select(
                 'jsonschema.collection', cnx, etype='Author')
-            jschema = collection_mapper.jsl_field(VIEW_ROLE).get_schema()
+            jschema = collection_mapper.json_schema(VIEW_ROLE)
         jsonschema.validate(related, jschema)
         self.assertEqual(related, expected)
 
-    def test_post_related_bad_identifier(self):
+    def test_post_related_not_an_object(self):
         with self.admin_access.cnx() as cnx:
-            book = cnx.create_entity('Book', title=u'title')
+            book = cnx.create_entity(
+                'Book', title=u'title',
+                author=cnx.create_entity('Author', name=u'a'))
             cnx.commit()
         self.login()
-        url = '/book/%s/author' % book.eid
-        res = self.webapp.post_json(url, ['a'], status=400,
+        url = '/book/%s/topics' % book.eid
+        res = self.webapp.post_json(url, ['ahah'], status=400,
                                     headers={'accept': 'application/json'})
-        self.assertIn('invalid target identifier(s)', res.json['message'])
+        expected = {
+            'status': 400,
+            'title': 'Bad Request',
+            'detail': 'expecting an object',
+        }
+        self.assertEqual(res.json, expected)
 
-    def test_post_related_bad_role(self):
+    def test_post_related_bad_identifier(self):
+        with self.admin_access.cnx() as cnx:
+            book = cnx.create_entity(
+                'Book', title=u'title',
+                author=cnx.create_entity('Author', name=u'a'))
+            cnx.create_entity('Topic', name=u'top')
+            cnx.commit()
+        self.login()
+        url = '/book/%s/topics' % book.eid
+        res = self.webapp.post_json(url, {'id': 'a'}, status=422,
+                                    headers={'accept': 'application/json'})
+        expected = {
+            'status': 422,
+            'title': 'Unprocessable Entity',
+            'invalid-params': [{
+                'name': 'topics',
+                'reason': 'value should be an array of string-encoded integers',
+            }],
+        }
+        self.assertEqual(res.json, expected)
+
+    def test_post_related_reverse_role(self):
         with self.admin_access.cnx() as cnx:
             author = cnx.create_entity('Author', name=u'bob')
             cnx.commit()
         self.login()
-        url = '/author/%s/author' % author.eid
-        res = self.webapp.post_json(url, [], status=404,
-                                    headers={'accept': 'application/json'})
-        self.assertIn('relation author-subject not found on Author',
-                      res.json['message'])
+        url = '/author/%s/reverse-author' % author.eid
+        data = {'title': 'my first book'}
+        response = self.webapp.post_json(
+            url, data, status=201,
+            headers={'accept': 'application/json'})
+        expected_instance = {
+            'title': 'my first book',
+            'author': [{'id': str(author.eid)}],
+        }
+        self.assertEqual(response.json, expected_instance)
+        with self.admin_access.cnx() as cnx:
+            rset = cnx.find('Book', title=u'my first book')
+            self.assertTrue(rset)
+            book = rset.one()
+        expected = 'https://localhost:80/author/{}/reverse-author/{}/'.format(
+            author.eid, book.eid)
+        self.assertEqual(response.location, expected)
+        # Test GET on related entity (could be a dedicated test).
+        response = self.webapp.get(response.location, status=200,
+                                   headers={'Accept': 'application/json'})
+        self.assertEqual(response.json, expected_instance)
+
+    def test_get_related_reverse_role(self):
+        with self.admin_access.cnx() as cnx:
+            author = cnx.create_entity('Author', name=u'bob')
+            books = [
+                cnx.create_entity('Book', title=u'1', author=author),
+                cnx.create_entity('Book', title=u'2', author=author),
+            ]
+            cnx.commit()
+        self.login()
+        url = '/author/%s/reverse-author' % author.eid
+        response = self.webapp.get(url, status=200,
+                                   headers={'Accept': 'application/json'})
+        expected = [
+            {'id': str(entity.eid),
+             'title': entity.dc_title(),
+             'type': entity.cw_etype.lower()}
+            for entity in books
+        ]
+        self.assertCountEqual(response.json, expected)
 
     def test_post_related_entity_notfound(self):
         self.login()
-        url = '/book/999/author'
-        self.webapp.post_json(url, [], status=404,
+        url = '/book/999/topics'
+        self.webapp.post_json(url, {}, status=404,
                               headers={'accept': 'application/json'})
 
     def test_post_related_bad_target(self):
         with self.admin_access.cnx() as cnx:
-            book = cnx.create_entity('Book', title=u'title')
+            book = cnx.create_entity(
+                'Book', title=u'title',
+                author=cnx.create_entity('Author', name=u'a'))
             cnx.commit()
             cwuser_eid = cnx.find('CWUser', login=u'admin')[0][0]
         self.login()
-        url = '/book/%s/author' % book.eid
+        url = '/book/%s/topics' % book.eid
         # TODO: status should be 4xx
-        self.webapp.post_json(url, [str(cwuser_eid)], status=500,
+        self.webapp.post_json(url, {'id': str(cwuser_eid)}, status=500,
                               headers={'accept': 'application/json'})
 
     def test_put_with_incomplete_data(self):
@@ -191,7 +288,8 @@ class EntitiesTC(BaseTC):
             )
             cnx.commit()
 
-        ascending = [book1.title, book2.title]
+        self.login('anon', 'anon')
+        ascending = [str(book1.eid), str(book2.eid)]
         descending = ascending[::-1]
         for sort, expected in [
             ('publication_date', ascending),
@@ -204,70 +302,9 @@ class EntitiesTC(BaseTC):
                     url, headers={'accept': 'application/json'})
                 entities = res.json
                 self.assertEqual(len(entities), 2)
-                ids = [d['title'] for d in entities]
+                ids = [d['id'] for d in entities]
                 self.assertEqual(ids, expected)
 
-    def test_add_related(self):
-        """POST on /<etype>/<eid>/relationships/<rtype> with primary entity as
-        subject of <rtype>.
-        """
-        with self.admin_access.repo_cnx() as cnx:
-            book = cnx.create_entity('Book', title=u'tmp')
-            cnx.commit()
-        url = '/book/%d/relationships/author' % book.eid
-        data = {
-            'name': u'bob',
-        }
-        self.login()
-        res = self.webapp.post_json(url, data, status=201,
-                                    headers={'accept': 'application/json'})
-        entity = res.json
-        with self.admin_access.cnx() as cnx:
-            author_eid = cnx.find('Author').one().eid
-            jschema = jsonschema_adapter(
-                cnx, etype='Author').view_schema()
-        self.assertEqual(
-            res.location,
-            'https://localhost:80/author/{}/'.format(author_eid))
-        jsonschema.validate(entity, jschema)
-        self.assertEqual(entity['name'], u'bob')
-        with self.admin_access.cnx() as cnx:
-            rset = cnx.execute(
-                'Any A WHERE B author A, B eid %(b)s, A name "bob"',
-                {'b': book.eid})
-        self.assertTrue(rset)
-
-    def test_add_related_object(self):
-        """POST on /<etype>/<eid>/relationships/<rtype> with primary entity as
-        object of <rtype>.
-        """
-        with self.admin_access.repo_cnx() as cnx:
-            author = cnx.create_entity('Author', name=u'bob')
-            cnx.commit()
-        url = '/author/%s/relationships/author' % author.eid
-        data = {
-            'title': u'introducing cubicweb-jsonschema',
-        }
-        self.login()
-        res = self.webapp.post_json(url, data, status=201,
-                                    headers={'accept': 'application/json'})
-        entity = res.json
-        with self.admin_access.cnx() as cnx:
-            book_eid = cnx.find('Book').one().eid
-            jschema = jsonschema_adapter(
-                cnx, etype='Book').view_schema()
-        self.assertEqual(
-            res.location,
-            'https://localhost:80/book/{}/'.format(book_eid))
-        jsonschema.validate(entity, jschema)
-        self.assertEqual(entity['title'], u'introducing cubicweb-jsonschema')
-        with self.admin_access.cnx() as cnx:
-            rset = cnx.execute(
-                'Any X WHERE X author A, A eid %(a)s,'
-                ' X title "introducing cubicweb-jsonschema"',
-                {'a': author.eid})
-        self.assertTrue(rset)
-
     def test_validationerror_additionalproperties(self):
         data = {
             u'name': u'bob',
@@ -290,7 +327,7 @@ class EntitiesTC(BaseTC):
         data = {
             'login': u'bob',
             'upassword': u'pass',
-            'in_group': [str(gid)],
+            'in_group': [{'id': str(gid)}],
         }
         with patch('cubicweb.req.RequestSessionBase.create_entity',
                    side_effect=ValidationError(None, {None: u'unmapped'})):
@@ -312,7 +349,7 @@ class EntitiesTC(BaseTC):
         data = {
             'login': 'bob',
             'upassword': 'pass',
-            'in_group': [str(gid)],
+            'in_group': [{'id': str(gid)}],
         }
         self.login()
         res = self.webapp.post_json('/cwuser/', data, status=422,
@@ -333,25 +370,26 @@ class EntitiesTC(BaseTC):
         data = {
             'login': 'bob',
             'upassword': 'pass',
-            'in_group': [str(gid)],
+            'in_group': [{'id': str(gid)}],
         }
         self.login()
-        error = ValueError("invalid literal for int() with base 10: 'foo'")
+        msg = "invalid literal for int() with base 10: 'foo'"
         with patch('cubicweb.req.RequestSessionBase.create_entity',
-                   side_effect=error):
+                   side_effect=ValueError(msg)):
             res = self.webapp.post_json('/cwuser/', data, status=500,
                                         headers={'Accept': 'application/json'})
             self.assertEqual(res.content_type, 'application/problem+json')
             self.assertEqual(res.json_body, {
                 u'status': 500,
                 u'title': u'Internal Server Error',
-                u'detail': u"invalid literal for int() with base 10: 'foo'",
+                u'detail': u"ValueError: %s" % msg,
             })
 
     def test_exception_no_detail(self):
         data = {'title': 'thing'}
         self.login()
-
+        # "detail" is only present in test/debug mode.
+        self.config.mode = 'dummy'
         with patch('cubicweb.req.RequestSessionBase.create_entity',
                    side_effect=NotImplementedError()):
             res = self.webapp.post_json('/book/', data, status=500,
@@ -362,6 +400,104 @@ class EntitiesTC(BaseTC):
                 u'title': u'Internal Server Error',
             })
 
+    def test_pagination(self):
+        with self.admin_access.cnx() as cnx:
+            eids = [cnx.create_entity('Author', name=name).eid
+                    for name in [u'Bob', u'Alice', u'Eve', u'John']]
+            cnx.commit()
+
+        self.login()
+        res = self.webapp.get('/author/?limit=1',
+                              headers={'Accept': 'application/json'})
+
+        self.assertEqual([int(x["id"]) for x in res.json_body], [eids[-1]])
+
+        links = parse_links(res.headers['Link'])
+
+        self.assertEqual(links.getone('next'), {
+            'href': '/author/?limit=1&offset=1',
+            'type': 'application/json',
+        })
+        self.assertNotIn('prev', links)
+
+        res = self.webapp.get('/author/?limit=1&offset=1',
+                              headers={'Accept': 'application/json'})
+        links = parse_links(res.headers['Link'])
+        self.assertEqual([int(x["id"]) for x in res.json_body], [eids[-2]])
+
+        expected_links = MultiDict({
+            'next': {
+                'href': '/author/?limit=1&offset=2',
+                'type': 'application/json',
+            },
+            'prev': {
+                'href': '/author/?limit=1&offset=0',
+                'type': 'application/json',
+            },
+        })
+        self.assertDictContainsSubset(expected_links, links)
+
+        # last page
+        res = self.webapp.get('/author/?limit=1&offset=3',
+                              headers={'Accept': 'application/json'})
+        links = parse_links(res.headers['Link'])
+        self.assertEqual([int(x["id"]) for x in res.json_body], [eids[0]])
+
+        expected_links = MultiDict({
+            'prev': {
+                'href': '/author/?limit=1&offset=2',
+                'type': 'application/json',
+            },
+        })
+        self.assertDictContainsSubset(expected_links, links)
+        self.assertNotIn('next', links)
+
+    def test_pagination_badrequest(self):
+        self.login()
+
+        res = self.webapp.get('/author/?limit=0', status=400,
+                              headers={'Accept': 'application/json'})
+        errors = res.json_body
+        expected = {
+            'status': 400,
+            'detail': 'invalid limit: 0',
+            'title': 'Bad Request',
+        }
+        self.assertEqual(errors, expected)
+
+        res = self.webapp.get('/author/?limit=foo', status=400,
+                              headers={'Accept': 'application/json'})
+        errors = res.json_body
+        expected = {
+            'status': 400,
+            'detail': "invalid limit (foo)",
+            'title': 'Bad Request',
+        }
+        self.assertEqual(errors, expected)
+
+
+class BWCompatTC(BaseTC):
+    """Tests with 'cubicweb.bwcompat = true'."""
+
+    settings = BaseTC.settings.copy()
+    settings['cubicweb.bwcompat'] = True
+
+    def test_cubicweb_requesterror(self):
+        self.login()
+        # Raises RemoteCallFailed (which has status=500).
+        response = self.webapp.post_json('/ajax?fname=format_date', status=500,
+                                         headers={'Accept': 'application/json'})
+        self.assertEqual(response.content_type, 'application/problem+json')
+        if six.PY2:
+            detail = 'TypeError: format_date() takes exactly 2 arguments (1 given)'  # noqa: E501
+        else:
+            detail = 'TypeError: format_date() missing 1 required positional argument: \'strdate\''  # noqa: E501
+        self.assertEqual(response.json_body, {
+            'status': 500,
+            'title': 'Internal Server Error',
+            'detail': detail,
+        })
+
 
 if __name__ == '__main__':
     import unittest
