@@ -1,10 +1,12 @@
 """Tests for "testing" module."""
+from __future__ import print_function
 
 from functools import partial
 import doctest
 import unittest
 
 import webtest
+from six import PY2
 
 from cubicweb_jsonschema import testing
 
@@ -26,9 +28,12 @@ class WebTestReponseOutputCheckerTC(unittest.TestCase):
             'Content-Type: application/json',
             '{"id": "1", "login": "alice"}',
         ])
-        self.assertTrue(self.checker.check_output(want, got, 0))
+        self.assertTrue(self.checker.check_output(want, got,
+                                                  testing.WEBTEST_RESPONSE))
         self.assertMultiLineEqual(
-            self.checker.output_difference(self.example(want), got, 0), '')
+            self.checker.output_difference(self.example(want), got,
+                                           testing.WEBTEST_RESPONSE),
+            '')
 
     def test_ok_json_body_prettyformat(self):
         got = '\n'.join([
@@ -44,9 +49,12 @@ class WebTestReponseOutputCheckerTC(unittest.TestCase):
             '  "login": "alice"',
             '}',
         ])
-        self.assertTrue(self.checker.check_output(want, got, 0))
+        self.assertTrue(self.checker.check_output(want, got,
+                                                  testing.WEBTEST_RESPONSE))
         self.assertMultiLineEqual(
-            self.checker.output_difference(self.example(want), got, 0), '')
+            self.checker.output_difference(self.example(want), got,
+                                           testing.WEBTEST_RESPONSE),
+            '')
 
     def test_json_ellipsis_object(self):
         got = '\n'.join([
@@ -62,8 +70,10 @@ class WebTestReponseOutputCheckerTC(unittest.TestCase):
             '  "login": "alice"'
             '}',
         ])
-        self.assertFalse(self.checker.check_output(want, got, 0))
-        self.assertTrue(self.checker.check_output(want, got, doctest.ELLIPSIS))
+        flags = testing.WEBTEST_RESPONSE
+        self.assertFalse(self.checker.check_output(want, got, flags))
+        flags |= doctest.ELLIPSIS
+        self.assertTrue(self.checker.check_output(want, got, flags))
 
     def test_json_ellipsis_list(self):
         got = '\n'.join([
@@ -78,8 +88,10 @@ class WebTestReponseOutputCheckerTC(unittest.TestCase):
             '  {"id": "...", "login": "alice"}'
             ']',
         ])
-        self.assertFalse(self.checker.check_output(want, got, 0))
-        self.assertTrue(self.checker.check_output(want, got, doctest.ELLIPSIS))
+        flags = testing.WEBTEST_RESPONSE
+        self.assertFalse(self.checker.check_output(want, got, flags))
+        flags |= doctest.ELLIPSIS
+        self.assertTrue(self.checker.check_output(want, got, flags))
 
     def test_bad_json(self):
         got = '\n'.join([
@@ -92,12 +104,14 @@ class WebTestReponseOutputCheckerTC(unittest.TestCase):
             'Content-Type: application/json',
             '{"id": "1"}',
         ])
-        self.assertFalse(self.checker.check_output(want, got, 0))
+        self.assertFalse(self.checker.check_output(want, got,
+                                                   testing.WEBTEST_RESPONSE))
         expected = '\n'.join([
             'Actual body is invalid JSON: {I am invalid JSON!',
         ])
         self.assertMultiLineEqual(
-            self.checker.output_difference(self.example(want), got, 0),
+            self.checker.output_difference(self.example(want), got,
+                                           testing.WEBTEST_RESPONSE),
             expected)
 
     def test_status_differ(self):
@@ -114,13 +128,15 @@ class WebTestReponseOutputCheckerTC(unittest.TestCase):
             '  "login": "alice"',
             '}',
         ])
-        self.assertFalse(self.checker.check_output(want, got, 0))
+        self.assertFalse(self.checker.check_output(want, got,
+                                                   testing.WEBTEST_RESPONSE))
         expected = '\n'.join([
             'Expected status: Response: 404 Not Found',
             'Actual status: Response: 200 OK',
         ])
         self.assertMultiLineEqual(
-            self.checker.output_difference(self.example(want), got, 0),
+            self.checker.output_difference(self.example(want), got,
+                                           testing.WEBTEST_RESPONSE),
             expected)
 
     def test_headers_differ(self):
@@ -134,13 +150,15 @@ class WebTestReponseOutputCheckerTC(unittest.TestCase):
             'Content-Type: application/json',
             '{"x": 1}',
         ])
-        self.assertFalse(self.checker.check_output(want, got, 0))
+        self.assertFalse(self.checker.check_output(want, got,
+                                                   testing.WEBTEST_RESPONSE))
         expected = '\n'.join([
             'Expected headers: Content-Type: application/json',
             'Actual headers: Content-Type: application/octet-stream',
         ])
         self.assertMultiLineEqual(
-            self.checker.output_difference(self.example(want), got, 0),
+            self.checker.output_difference(self.example(want), got,
+                                           testing.WEBTEST_RESPONSE),
             expected)
 
     def test_headers_missing_from_got(self):
@@ -155,7 +173,8 @@ class WebTestReponseOutputCheckerTC(unittest.TestCase):
             'Link: </schema> rel="describedby"',
             '{"x": 1}',
         ])
-        self.assertFalse(self.checker.check_output(want, got, 0))
+        self.assertFalse(self.checker.check_output(want, got,
+                                                   testing.WEBTEST_RESPONSE))
 
     def test_headers_extra_in_got(self):
         got = '\n'.join([
@@ -169,7 +188,8 @@ class WebTestReponseOutputCheckerTC(unittest.TestCase):
             'Content-Type: application/json',
             '{"x": 1}',
         ])
-        self.assertTrue(self.checker.check_output(want, got, 0))
+        self.assertTrue(self.checker.check_output(want, got,
+                                                  testing.WEBTEST_RESPONSE))
 
     def test_headers_ellipsis(self):
         got = '\n'.join([
@@ -184,16 +204,17 @@ class WebTestReponseOutputCheckerTC(unittest.TestCase):
             'Location: http://www.example.com/...',
             '{"x": 1}',
         ])
-        self.assertFalse(self.checker.check_output(want, got, 0))
-        self.assertTrue(self.checker.check_output(want, got, doctest.ELLIPSIS))
+        flags = testing.WEBTEST_RESPONSE
+        self.assertFalse(self.checker.check_output(want, got, flags))
+        flags |= doctest.ELLIPSIS
+        self.assertTrue(self.checker.check_output(want, got, flags))
         want = '\n'.join([
             'Response: 200 OK',
             'Content-Type: application/json',
             'Location: http://www.example.org/...',
             '{"x": 1}',
         ])
-        self.assertFalse(self.checker.check_output(
-            want, got, doctest.ELLIPSIS))
+        self.assertFalse(self.checker.check_output(want, got, flags))
 
     def test_headers_ellipsis_body_diff(self):
         got = '\n'.join([
@@ -209,8 +230,9 @@ class WebTestReponseOutputCheckerTC(unittest.TestCase):
             '{"id": "2"}',
         ])
         self.assertFalse(self.checker.check_output(
-            want, got, doctest.ELLIPSIS))
-        difference = self.checker.output_difference(self.example(want), got, 0)
+            want, got, testing.WEBTEST_RESPONSE | doctest.ELLIPSIS))
+        difference = self.checker.output_difference(self.example(want), got,
+                                                    testing.WEBTEST_RESPONSE)
         # Do not check full content until output_difference() handles ELLIPSIS
         # flag.
         self.assertIn('JSON bodies differ', difference)
@@ -226,9 +248,10 @@ class WebTestReponseOutputCheckerTC(unittest.TestCase):
             'Content-Type: application/json',
             '{"id": "2"}',
         ])
-        self.assertFalse(self.checker.check_output(want, got, 0))
+        self.assertFalse(self.checker.check_output(want, got,
+                                                   testing.WEBTEST_RESPONSE))
         self.assertFalse(self.checker.check_output(
-            want, got, doctest.ELLIPSIS))
+            want, got, testing.WEBTEST_RESPONSE | doctest.ELLIPSIS))
         expected = '\n'.join([
             'JSON bodies differ',
             'expected',
@@ -240,21 +263,30 @@ class WebTestReponseOutputCheckerTC(unittest.TestCase):
             '  "id": "1"',
             '}',
             'diff:',
-            '{"id": ["2", "1"]}',
+            '{',
+            '  "id": [',
+            '    "2",' + (' ' if PY2 else ''),
+            '    "1"',
+            '  ]',
+            '}',
         ])
-        difference = self.checker.output_difference(self.example(want), got, 0)
+        difference = self.checker.output_difference(self.example(want), got,
+                                                    testing.WEBTEST_RESPONSE)
         self.assertMultiLineEqual(difference, expected)
 
     def test_no_content(self):
         got = 'Response: 204 No Content<BLANKLINE><BLANKLINE>'
         want = 'Response: 204 No Content<BLANKLINE>'
-        self.assertTrue(self.checker.check_output(want, got, 0))
+        self.assertTrue(self.checker.check_output(want, got,
+                                                  testing.WEBTEST_RESPONSE))
         got = 'Response: 204 No Content\nFoo: bar\n'
         want = 'Response: 204 No Content\nFoo: bar\n{"x": 1}'
-        self.assertFalse(self.checker.check_output(want, got, 0))
+        self.assertFalse(self.checker.check_output(want, got,
+                                                   testing.WEBTEST_RESPONSE))
         got = 'Response: 204 No Content\nFoo: bar\n{"x": 1}'
         want = 'Response: 204 No Content\nFoo: bar\n<BLANKLINE>'
-        self.assertFalse(self.checker.check_output(want, got, 0))
+        self.assertFalse(self.checker.check_output(want, got,
+                                                   testing.WEBTEST_RESPONSE))
 
     def test_parse_error(self):
         got = '\n'.join([
@@ -265,12 +297,14 @@ class WebTestReponseOutputCheckerTC(unittest.TestCase):
             'Content-Type: something',
             'bobybody',
         ])
-        self.assertFalse(self.checker.check_output(want, got, 0))
+        self.assertFalse(self.checker.check_output(want, got,
+                                                   testing.WEBTEST_RESPONSE))
         expected = '\n'.join([
             'In expected: not a webtest.TestResponse string', want,
             'In actual: not a webtest.TestResponse string', got,
         ])
-        difference = self.checker.output_difference(self.example(want), got, 0)
+        difference = self.checker.output_difference(self.example(want), got,
+                                                    testing.WEBTEST_RESPONSE)
         self.assertMultiLineEqual(difference, expected)
 
 
@@ -298,7 +332,9 @@ def unittest_integration_test():
 
     return doctest.DocFileSuite(
         'doctest-example.txt', setUp=setUp,
-        checker=testing.WebTestReponseOutputChecker())
+        checker=testing.WebTestReponseOutputChecker(),
+        optionflags=testing.WEBTEST_RESPONSE,
+    )
 
 
 def load_tests(loader, tests, ignore):
