@@ -12,7 +12,7 @@
 # details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with this program. If not, see <https://www.gnu.org/licenses/>.
+# along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 """cubicweb-jsonschema views tests"""
 
@@ -42,6 +42,8 @@ class JSONSchemaRelationTagsTC(CubicWebTC):
             # plain relation
             (('Book', 'topics', 'Topic', 'subject'), 'related'),
             (('Book', 'topics', 'Topic', 'object'), 'related'),
+            (('Book', 'author', 'Author', 'subject'), 'inlined'),
+            (('Book', 'in_library', 'Library', 'subject'), 'related'),
             # computed relation
             (('Author', 'publications', 'Book', 'object'), 'related'),
         ):
@@ -86,10 +88,15 @@ class JSONSchemaRelationTagsTC(CubicWebTC):
         with self.admin_access.cnx() as cnx:
             entity = self.fake_entity(cnx, 'Book')
             rsection = self.rtag(cnx, entity)
-            rtypes = [
+            expected = ['title', 'publication_date', 'author']
+            read_rtypes = [
                 rtype for rtype, _, _ in rsection.relations_by_section(
                     entity, 'inlined', 'read')]
-            self.assertCountEqual(['title', 'publication_date'], rtypes)
+            self.assertCountEqual(read_rtypes, expected)
+            add_rtypes = [
+                rtype for rtype, _, _ in rsection.relations_by_section(
+                    entity, 'inlined', 'add')]
+            self.assertCountEqual(add_rtypes, expected)
 
     def test_relations_by_section_related(self):
         with self.admin_access.cnx() as cnx:
@@ -97,12 +104,13 @@ class JSONSchemaRelationTagsTC(CubicWebTC):
             rsection = self.rtag(cnx, entity)
             rtypes = [rtype for rtype, _, _ in rsection.relations_by_section(
                 entity, 'related', 'read')]
-            self.assertCountEqual(['author', 'publications', 'topics'],
+            self.assertCountEqual(['in_library', 'publications', 'topics'],
                                   rtypes)
 
     def test_relations_by_section_inlined_permissions(self):
         with self.admin_access.cnx() as cnx:
-            cnx.create_entity('Book', title=u't')
+            cnx.create_entity('Book', title=u't',
+                              author=cnx.create_entity('Author', name=u'a'))
             self.create_user(cnx, u'bob')
             cnx.commit()
         with self.new_access(u'bob').cnx() as cnx:
@@ -111,11 +119,12 @@ class JSONSchemaRelationTagsTC(CubicWebTC):
             read_rtypes = [
                 rtype for rtype, _, _ in rsection.relations_by_section(
                     entity, 'inlined', 'read')]
-            self.assertCountEqual(['title', 'publication_date'], read_rtypes)
+            self.assertCountEqual(['title', 'publication_date', 'author'],
+                                  read_rtypes)
             update_rtypes = [
                 rtype for rtype, _, _ in rsection.relations_by_section(
                     entity, 'inlined', 'update')]
-            self.assertCountEqual([], update_rtypes)
+            self.assertCountEqual(['author'], update_rtypes)
 
     def test_relations_by_section_related_permissions(self):
         with self.admin_access.cnx() as cnx:
@@ -129,7 +138,12 @@ class JSONSchemaRelationTagsTC(CubicWebTC):
             update_rtypes = [
                 rtype for rtype, _, _ in rsection.relations_by_section(
                     entity, 'related', 'add')]
-            self.assertCountEqual(['topics'],
+            self.assertCountEqual(['topics', 'in_library'],
+                                  update_rtypes)
+            update_rtypes = [
+                rtype for rtype, _, _ in rsection.relations_by_section(
+                    entity, 'inlined', 'add')]
+            self.assertCountEqual(['title', 'publication_date', 'author'],
                                   update_rtypes)
 
 
